/******************************************************************************
* Function name         :  system.c
* returns               :  
* Created by            :  JBE, BHF
* Date Created          :  
* Company				:  Memec
* Description           :  This program tests the flash by doing the following:
*                       :  1) Erase the device; 2) Write Sample Data; 3) Readback
*                       :  and compare sample data
* Notes                 :  
******************************************************************************/
#include "xparameters.h"
#include "xuartlite_l.h"
#include "xgpio_l.h"
#include "stdio.h"

#define FLASHBASEADDR  XPAR_FLASH_2MX16_MEM0_BASEADDR
#define FLASHBYTES     (XPAR_FLASH_2MX16_MEM0_HIGHADDR - XPAR_FLASH_2MX16_MEM0_BASEADDR + 1)
#define FLASHHWORDS    (FLASHBYTES / 2)
#define FLASHWORDS     (FLASHBYTES / 4)

#define LEDBASEADDR    XPAR_LEDS_4BIT_BASEADDR

#define TEST_WORDS     FLASHWORDS
#define TEST_HWORDS    TEST_WORDS * 2
#define TEST_BYTES     TEST_WORDS * 4

#define WAIT_LENGTH			3000000

void _wait(loop_count)
int	loop_count;
{

int	sum, data;

sum	= 0;

for (data = 0; data < loop_count; data++) {
	sum = (data << 8);
}
  return;
}

int main(void)
{
	int initial_value, loop_count;
	int	i, j, sw_input, temp_data;

	unsigned int* sdram_location = (unsigned int *) XPAR_DDR_SDRAM_16MX16_MEM0_BASEADDR;
	int sdram_data_read, sdram_data_read_error;
	unsigned int* temp;

	int DIPswitch, PUSH, testmask;
	int DataWord, TestWord;
	char error = 0;
	int flash_rdy, prev_flash_rdy;

xil_printf("         ####################################################\n\r");
xil_printf("         #                                                  #\n\r");
xil_printf("         #                        Memec                     #\n\r");
xil_printf("         #            Spartan-3 MB Development Board        #\n\r");
xil_printf("         #                  EDK 7.1 Flash Test              #\n\r");
xil_printf("         ####################################################\n\r");
xil_printf("\n\r");
xil_printf("\n\r");

//	xil_printf("FLASHBYTES = : %x \n\r", FLASHBYTES);
//	xil_printf("FLASHHWORDS = : %x \n\r", FLASHHWORDS);
//	xil_printf("FLASHWORDS = : %x \n\r", FLASHWORDS);

	//--------------------------------------------------------------
	XGpio_mSetDataDirection(LEDBASEADDR, 1,0x00); 

	xil_printf("Set Config Register\n\r");
	// 4-cycle Config Register Command
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00AA);
	XIo_Out16(FLASHBASEADDR + (0xAAA << 1),0x0055);
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00D0);
	XIo_Out16(FLASHBASEADDR               ,0x0001);

// Get Product ID

	xil_printf("Flash ID Test\n\r");
	// 3-cycle Product ID Entry Command
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00AA);
	XIo_Out16(FLASHBASEADDR + (0xAAA << 1),0x0055);
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x0090);

	// Manufacturer Code is read for A0 = low
	// Expecting 0x1F00C8
	DataWord = XIo_In32(FLASHBASEADDR + 0x0);
	xil_printf("Manufacturer/Device Code = : %x \n\r", DataWord);
	if (DataWord == 0x1F00C8)
		xil_printf("ID Test passed.\n\n\r");
	else
		xil_printf("ID Test failed.\n\n\r");

	// 1-cycle Product ID Exit Command
	XIo_Out16(FLASHBASEADDR,0x00F0);

// Perform a Chip Erase
	xil_printf("Chip Erase Starting -- 50 seconds max\n\r");

	// 6-cycle Chip Erase Command
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00AA);
	XIo_Out16(FLASHBASEADDR + (0xAAA << 1),0x0055);
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x0080);
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00AA);
	XIo_Out16(FLASHBASEADDR + (0xAAA << 1),0x0055);
	XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x0010);

	// Check until the status bit is High
	do
	{
		xil_printf(".");
		_wait(WAIT_LENGTH);
		flash_rdy = XIo_In16(FLASHBASEADDR) & (0x0080);
	}
	while (!flash_rdy);

	// 1-cycle Product ID Exit Command
	XIo_Out16(FLASHBASEADDR,0x00F0);

	xil_printf("\n\rChip Erase Complete - verifying\n\r");

// Verify Chip Erase
	j = FLASHBYTES;
	for (i = 0; i < FLASHWORDS; i++)
	{
		DataWord = XIo_In32(FLASHBASEADDR + (i << 2));
		if (DataWord != 0xFFFFFFFF)
		{
			xil_printf("Chip Erase Failed at %x\n\r", i);
			xil_printf("Data read was %x\n\r",DataWord);
			error = 1;
			i = FLASHWORDS;
		}
	}
	if(!error)
		xil_printf("Chip Erase succeeded\n\n\r");

// Write Test using Word Program mode
xil_printf("Data being written -- 20 seconds max\n\r");
	for (i = 0; i < TEST_BYTES; i=i+2)
	{
		XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00AA);
		XIo_Out16(FLASHBASEADDR + (0xAAA << 1),0x0055);
		XIo_Out16(FLASHBASEADDR + (0x555 << 1),0x00A0);
		XIo_Out16(FLASHBASEADDR + i,i);

		// Check until the status bit is High
		do
			flash_rdy = XIo_In16(FLASHBASEADDR) & (0x0080);
		while (!flash_rdy);
	}

	// 1-cycle Product ID Exit Command
	XIo_Out16(FLASHBASEADDR,0x00F0);

	xil_printf("Test Data Written; reading back\n\r", i);
	
	// Readback all bytes
	error = 0;
	for (i = 0; i < TEST_BYTES; i=i+4)
	{
		DataWord = XIo_In32(FLASHBASEADDR + i);
		//xil_printf("%x Data read was %x\n\r",i,DataWord);
		TestWord = ((i<<16) | ((i & 0xFFFF) + 2));
		if (DataWord != TestWord)
		{
			xil_printf("Flash Test FAILED at %x\n\r", i);
			xil_printf("Data read was %x\n\r",DataWord);
			xil_printf("Exp. Data was %x\n\r",TestWord);
			error = 1;
			i = TEST_BYTES;
		}

	}
	if(!error)
	{
		xil_printf("Flash Test PASSED!\n\r");
		//xil_printf("Last Address Read was %x\n\r", i-4);
		//xil_printf("Last Data Read was %x\n\r", DataWord);
	}

	//--------------------------------------------------------------
	while(1)
	{
		_wait(WAIT_LENGTH);
		XGpio_mSetDataReg(LEDBASEADDR, 1,0xFF);
		_wait(WAIT_LENGTH);
		XGpio_mSetDataReg(LEDBASEADDR, 1,0xFE);
		_wait(WAIT_LENGTH);
		XGpio_mSetDataReg(LEDBASEADDR, 1,0xFD);
		_wait(WAIT_LENGTH);
		XGpio_mSetDataReg(LEDBASEADDR, 1,0xFB);
		_wait(WAIT_LENGTH);
		XGpio_mSetDataReg(LEDBASEADDR, 1,0xF7);
	}

	return 0;

}
