-- $Id: opb_core_ssp0_ref.vhd,v 1.1 2003/06/26 14:10:56 anitas Exp $
--USER-- Add comment with name of this source file
-- opb_core_ssp0_ref.vhd
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
--
-------------------------------------------------------------------------------
-- Filename:        opb_core_ssp0_ref.vhd
-- Version:         v1.00c
-- Description:     OPB Slave Services Package 0 Template
--                  Instantiates OPB IPIF SSP0 and example user logic
--
-------------------------------------------------------------------------------
-- Structure: 
--              opb_core_ssp0_ref.vhd
--                  -- opb_ipif_ssp0.vhd
--                  -- user_logic.vhd
--                      -- piezoer.vhd
--
-------------------------------------------------------------------------------
-- Author:      BLT
-- History:
--  ALS             06-24-2003      
-- ^^^^^^
--      Version C
-- ~~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library opb_ipif_ssp0_v1_00_b;
use opb_ipif_ssp0_v1_00_b.all;

--  Add user library if this source file uses
--  entities from the user library
library memec_piezo_v1_00_c; --USER-- library name
use memec_piezo_v1_00_c.all; --USER-- use statement

-------------------------------------------------------------------------------
-- Port Declarations
-------------------------------------------------------------------------------
-- Definition of Generics: 
--      C_BASEADDR                 -- User logic base address                                                                                 
--      C_HIGHADDR                 -- User logic high address
--      C_MIR_BASEADDR             -- Base address of MIR/Reset register
--      C_MIR_HIGHADDR             -- High address of MIR/Reset register
--      C_USER_ID_CODE             -- User ID to place in MIR/Reset register
--      C_OPB_DWIDTH               -- OPB data bus width
--      C_OPB_AWIDTH               -- OPB address bus width
--      C_FAMILY                   -- Target FPGA architecture
--
-- Definition of Ports:
--  OPB Bus
--      OPB_ABus                    -- OPB address bus                                             
--      OPB_BE                      -- OPB byte enables
--      OPB_Clk                     -- OPB clock
--      OPB_DBus                    -- OPB data bus                                                   
--      OPB_RNW                     -- OPB read not write 
--      OPB_Rst                     -- OPB reset
--      OPB_select                  -- OPB Select                                    
--      OPB_seqAddr                 -- OPB sequential address                              
--      Sln_DBus                    -- Slave read bus                                         
--      Sln_errAck                  -- Slave Error acknowledge
--      Sln_retry                   -- Slave retry
--      Sln_toutSup                 -- Slave Timeout Suppress
--      Sln_xferAck                 -- Slave transfer acknowledge   
-- User Logic
--      LED                         -- Outputs to the LEDs        
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Entity Section
-------------------------------------------------------------------------------

entity memec_piezo is  --USER-- change entity name
  generic
  (
    C_BASEADDR         : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_HIGHADDR         : std_logic_vector(0 to 31) := X"00000000";
    C_MIR_BASEADDR     : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_MIR_HIGHADDR     : std_logic_vector(0 to 31) := X"00000000";
    C_USER_ID_CODE     : integer                   := 3;
    C_OPB_AWIDTH       : integer                   := 32;
    C_OPB_DWIDTH       : integer                   := 32;
    C_FAMILY           : string                    := "virtex2" -- not used
  );
  port
  (
    --Required OPB bus ports, do not add to or delete
    OPB_ABus     : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE       : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk      : in  std_logic;
    OPB_DBus     : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW      : in  std_logic;
    OPB_Rst      : in  std_logic;
    OPB_select   : in  std_logic;
    OPB_seqAddr  : in  std_logic;
    Sln_DBus     : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    Sln_errAck   : out std_logic;
    Sln_retry    : out std_logic;
    Sln_toutSup  : out std_logic;
    Sln_xferAck  : out std_logic;

    --USER-- add user I/Os to port list
    PIEZO        : out std_logic

  );

      --fan-out attributes for XST
 attribute MAX_FANOUT                  : string;
 attribute MAX_FANOUT   of OPB_Clk     : signal is "10000";
 attribute MAX_FANOUT   of OPB_Rst     : signal is "10000";


end entity memec_piezo; --USER-- change entity name

-------------------------------------------------------------------------------
-- Architecture Section
-------------------------------------------------------------------------------

architecture imp of memec_piezo is --USER-- change entity name

-------------------------------------------------------------------------------
-- Constant Declarations
-------------------------------------------------------------------------------
--USER-- change PIPELINE_MODEL constant below to desired pipeline model for
--       user logic:
--          1 = include OPB-In pipeline registers
--          2 = include IP pipeline registers
--          3 = include OPB-In and IP pipeline registers
--          4 = include OPB-Out pipeline registers
--          5 = include OPB-In and OPB-Out pipeline registers
--          6 = include IP and OPB-Out pipeline registers
--          7 = include OPB-In, IP, and OPB-Out pipeline registers

constant PIPELINE_MODEL : integer   := 5;  -- example user logic uses 
                                           -- OPB-In and OPB-Out pipeline regs

--USER-- add constant declarations as needed for user core logic

-------------------------------------------------------------------------------
-- Signal and Type Declarations
-------------------------------------------------------------------------------
--IP Interconnect (IPIC) signal list --do not delete
signal Bus2IP_Addr        : std_logic_vector(0 to C_OPB_AWIDTH-1);
signal Bus2IP_BE          : std_logic_vector(0 to C_OPB_DWIDTH/8-1);
signal Bus2IP_Burst       : std_logic;
signal Bus2IP_Clk         : std_logic;
signal Bus2IP_CS          : std_logic;
signal Bus2IP_Data        : std_logic_vector(0 to C_OPB_DWIDTH-1);
signal Bus2IP_RdCE        : std_logic;
signal Bus2IP_Reset       : std_logic;
signal Bus2IP_RNW         : std_logic;
signal Bus2IP_WrCE        : std_logic;
signal IP2Bus_Ack         : std_logic;
signal IP2Bus_Data        : std_logic_vector(0 to C_OPB_DWIDTH-1);
signal IP2Bus_Error       : std_logic;
signal IP2Bus_PostedWrInh : std_logic;
signal IP2Bus_Retry       : std_logic;
signal IP2Bus_ToutSup     : std_logic;

--USER-- added signal declarations as needed for user logic         

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
--USER-- component declaration for user core logic. Change the
--component declaration below to the declaration of the user's
--logic.

component user_logic is
  port
  (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    PIEZO        : out std_logic
  );
end component user_logic;

--OPB IPIF slave services package 0 declaration -- do not change
component opb_ipif_ssp0 is
  generic
  (
    C_BASEADDR         : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_HIGHADDR         : std_logic_vector(0 to 31) := X"00000000";
    
    C_MIR_BASEADDR     : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_MIR_HIGHADDR     : std_logic_vector(0 to 31) := X"00000000";
    
    C_USER_ID_CODE     : integer                   := 1;
    C_PIPELINE_MODEL   : integer                   := 4;
    
    C_OPB_AWIDTH       : integer                   := 32;
    C_OPB_DWIDTH       : integer                   := 32;
    
    C_FAMILY           : string                    := "virtex2"
  );
  port
  (
      OPB_ABus           : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
      OPB_BE             : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      OPB_Clk            : in  std_logic;
      OPB_DBus           : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_RNW            : in  std_logic;
      OPB_Rst            : in  std_logic;
      OPB_select         : in  std_logic;
      OPB_seqAddr        : in  std_logic;
      Sln_DBus           : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      Sln_errAck         : out std_logic;
      Sln_retry          : out std_logic;
      Sln_toutSup        : out std_logic;
      Sln_xferAck        : out std_logic;
      Bus2IP_Addr        : out std_logic_vector(0 to C_OPB_AWIDTH-1);
      Bus2IP_BE          : out std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      Bus2IP_Burst       : out std_logic;
      Bus2IP_Clk         : out std_logic;
      Bus2IP_CS          : out std_logic;
      Bus2IP_Data        : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      Bus2IP_RdCE        : out std_logic;
      Bus2IP_Reset       : out std_logic;
      Bus2IP_RNW         : out std_logic;
      Bus2IP_WrCE        : out std_logic;
      IP2Bus_Ack         : in  std_logic;
      IP2Bus_Data        : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
      IP2Bus_Error       : in  std_logic;
      IP2Bus_PostedWrInh : in  std_logic;
      IP2Bus_Retry       : in  std_logic;
      IP2Bus_ToutSup     : in  std_logic
  );
end component opb_ipif_ssp0;

------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------

--OPB IPIF slave services package 0 --do not delete
OPB_IPIF_SSP0_I : opb_ipif_ssp0
  generic map
  (
    C_BASEADDR         => C_BASEADDR,
    C_HIGHADDR         => C_HIGHADDR,
    C_MIR_BASEADDR     => C_MIR_BASEADDR,
    C_MIR_HIGHADDR     => C_MIR_HIGHADDR,
    C_USER_ID_CODE     => C_USER_ID_CODE,
    C_PIPELINE_MODEL   => PIPELINE_MODEL,
    C_OPB_AWIDTH       => C_OPB_AWIDTH,
    C_OPB_DWIDTH       => C_OPB_DWIDTH,
    C_FAMILY           => C_FAMILY
  )
  port map
  (
    OPB_ABus           => OPB_ABus,
    OPB_BE             => OPB_BE,
    OPB_Clk            => OPB_Clk,
    OPB_DBus           => OPB_DBus,
    OPB_RNW            => OPB_RNW,
    OPB_Rst            => OPB_Rst,
    OPB_select         => OPB_select,
    OPB_seqAddr        => OPB_seqAddr,
    Sln_DBus           => Sln_DBus,
    Sln_errAck         => Sln_errAck,
    Sln_retry          => Sln_retry,
    Sln_toutSup        => Sln_toutSup,
    Sln_xferAck        => Sln_xferAck,
    Bus2IP_Addr        => Bus2IP_Addr,
    Bus2IP_BE          => Bus2IP_BE,
    Bus2IP_Burst       => Bus2IP_Burst,
    Bus2IP_Clk         => Bus2IP_Clk,
    Bus2IP_CS          => Bus2IP_CS,
    Bus2IP_Data        => Bus2IP_Data,
    Bus2IP_RdCE        => Bus2IP_RdCE,
    Bus2IP_Reset       => Bus2IP_Reset,
    Bus2IP_RNW         => Bus2IP_RNW,
    Bus2IP_WrCE        => Bus2IP_WrCE,
    IP2Bus_Ack         => IP2Bus_Ack,
    IP2Bus_Data        => IP2Bus_Data,
    IP2Bus_Error       => IP2Bus_Error,
    IP2Bus_PostedWrInh => IP2Bus_PostedWrInh,
    IP2Bus_Retry       => IP2Bus_Retry,
    IP2Bus_ToutSup     => IP2Bus_ToutSup
  );

--USER-- change the USER_LOGIC_I instantiation below to the
--instantiation of the user logic.
USER_LOGIC_I : user_logic
  port map
  (
    Bus2IP_Addr  => Bus2IP_Addr,
    Bus2IP_Clk   => Bus2IP_Clk,
    Bus2IP_CS    => Bus2IP_CS,
    Bus2IP_Data  => Bus2IP_Data,
    Bus2IP_Reset => Bus2IP_Reset,
    Bus2IP_RdCE  => Bus2IP_RdCE,
    Bus2IP_WrCE  => Bus2IP_WrCE,
    IP2Bus_Data  => IP2Bus_Data,
    PIEZO        => PIEZO
  );
  
--USER-- The following signals must be driven by the user_logic or by the 
--       following assignments.

-- no wait states, immediate Ack. Drive from user_logic to add wait states
IP2Bus_Ack         <= Bus2IP_WrCE or Bus2IP_RdCE; 
IP2Bus_Retry       <= '0'; -- no retry
IP2Bus_Error       <= '0'; -- no error
IP2Bus_ToutSup     <= '0'; -- no timeout suppress. If IP2Bus_Ack is delayed more than 15 clocks, drive
                           -- IP2Bus_ToutSup to '1' to avoid arbiter timeout on OPB.
IP2Bus_PostedWrInh <= '0'; -- do not inhibit posted write

end architecture imp;

