--SINGLE_FILE_TAG
-------------------------------------------------------------------------------
-- $Id: pwm.vhd,v 1.2 2003/06/27 00:56:18 anitas Exp $
-------------------------------------------------------------------------------
-- PWM - entity/architecture pair
-------------------------------------------------------------------------------
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        ssd.vhd
-- Version:         v1.00a
-- Description:     Pulse width modulator
--
-------------------------------------------------------------------------------
-- Structure: 
--
--              ssd.vhd
--
-------------------------------------------------------------------------------
-- Author:      JBE
-- History:
--  BLT             03-22-2004      -- First version
-- ^^^^^^
--      First version of ssd
-- ~~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_misc.all;

library Unisim;
use Unisim.Vcomponents.all;


-------------------------------------------------------------------------------
-- Port Declarations
-------------------------------------------------------------------------------
-- Definition of Ports:
--      PWM_In                     	-- hex input
--      PWM_Out                     -- ssd output
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Entity section
-------------------------------------------------------------------------------
entity PIEZOER is
    Port (  CLKIN : in std_logic;
		PIEZO_In : in std_logic_vector(0 to 7);
      	PIEZO_Out : out std_logic);
end PIEZOER;
-------------------------------------------------------------------------------
-- Architecture section
-------------------------------------------------------------------------------

architecture IMP of PIEZOER is

-------------------------------------------------------------------------------
-- Signal declarations
-------------------------------------------------------------------------------
signal CLK : std_logic;
signal DENOM : std_logic_vector(0 to 17);
signal COUNT : std_logic_vector(0 to 17);
-------------------------------------------------------------------------------
-- Attributes
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Begin architecture
-------------------------------------------------------------------------------

begin

--	PIEZO_In(0 to 7) =
--			CLK == 75 MHz / DENOM
--		
--			FF ==> DENOM = 261120 ==> CLK = 287.2
--
--			7F ==> DENOM = 130048 ==> CLK = 576.7 Hz
--			7E ==> DENOM = 129024 ==> CLK = 581.3 Hz
--
--			03 ==> DENOM = 3072 ==> 24414.1
--			02 ==> DENOM = 2048 ==> 36621.1
--			01 ==> DENOM = 1024 ==> 73242.2


DENOM <= PIEZO_In(0 to 7) & "0000000000";
-------------------------------------------------------------------------
process (CLKIN)
begin  
   if CLKIN='1' and CLKIN'event then
		if COUNT = DENOM then
	      CLK <= NOT CLK;
			COUNT <= "000000000000000000";
		else		
			COUNT <= COUNT + 1;
		end if;
   end if;
end process;
-------------------------------------------------------------------------
PIEZO_Out <= CLK; 
-------------------------------------------------------------------------
end architecture IMP;

