--SINGLE_FILE_TAG
-------------------------------------------------------------------------------
-- $Id: user_logic.vhd,v 1.1 2003/06/26 14:10:56 anitas Exp $
-------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
-------------------------------------------------------------------------------
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        user_logic.vhd
-- Version:         v1.00a
-- Description:     User Logic for the LED brightness control
--                  Contains 4 PWMs and 5 registers
--
-------------------------------------------------------------------------------
-- Structure: 
--
--              user_logic.vhd
--
-------------------------------------------------------------------------------
-- Author:      BLT
-- History:
--  BLT             10-22-2002      -- First version
-- ^^^^^^
--      First version of user_logic for LED brightness controller example.
-- ~~~~~~
--  ALS             06-24-2003      
-- ^^^^^^
--      Zero'd LSB of control vector to PWM (only allow even values) since 
--      differences in LSB weren't visible.
-- ~~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_misc.all;
library Unisim;
use Unisim.all;

-------------------------------------------------------------------------------
-- Port declarations
-------------------------------------------------------------------------------
-- Definition of Ports:
--  IPIC
--      Bus2IP_Addr                 -- Bus to IP address        
--      Bus2IP_Clk                  -- Bus to IP clock
--      Bus2IP_CS                   -- Bus to IP chip select
--      Bus2IP_Data                 -- Bus to IP data bus
--      Bus2IP_RdCE                 -- Bus to IP read chip enable
--      Bus2IP_Reset                -- Bus to IP reset
--      Bus2IP_WrCE                 -- Bus to IP write chip enable
--      IP2Bus_Data                 -- IP to Bus data bus
-- User Logic
--      PIEZO                       -- LED control outputs
-----------------------------------------------------------------------------
-- Entity section
-----------------------------------------------------------------------------

entity user_logic is
  port (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    PIEZO        : out std_logic
    );
end entity user_logic;

-------------------------------------------------------------------------------
-- Architecture section
-------------------------------------------------------------------------------

architecture IMP of user_logic is

-------------------------------------------------------------------------------
-- Signal declarations
-------------------------------------------------------------------------------
signal piezo_control_reg   : std_logic_vector(0 to 7);

-------------------------------------------------------------------------------
-- Component declarations
-------------------------------------------------------------------------------
component PIEZOER is
    Port 	(  
		CLKIN : in std_logic;
		PIEZO_In : in std_logic_vector(0 to 7);
      	PIEZO_Out : out std_logic
		);
end component PIEZOER;


-------------------------------------------------------------------------------
-- Begin architecture
-------------------------------------------------------------------------------

begin -- architecture IMP

REGS_PROC: process(Bus2IP_Clk) is
begin
  if Bus2IP_Clk'event and Bus2IP_Clk='1' then
    if Bus2IP_Reset = '1' then
      piezo_control_reg   <= (others => '0'); 
    else
      if Bus2IP_CS = '1' and Bus2IP_WrCE = '1' then 
          piezo_control_reg   <= Bus2IP_Data(24 to 31);
      end if;
    end if;
  end if;
end process REGS_PROC;

-- IP2Bus_Data(0 to 23) <= (others => '0');

-- instantiations
PIEZO_0: PIEZOER port map (CLKIN => Bus2IP_Clk, PIEZO_In => piezo_control_reg, PIEZO_Out => PIEZO); 

end architecture IMP;
